/*  smpbern.cpp  by K.Tsuru  */
const char* title = "<<< A sample program for tangent coefficient, Bernouilli and Euler numbers >>>\n";
#include "sn.h"

void TnTableTest(uint N)
{
	SNBlock <SLong> T(N+1);
	Timer timer;
	double cpu_time;

	cerr << "Calculating tangent coefficients by TanCoeffTable() function." << endl;
	timer.Start();
	TanCoeffTable(T, N);
	cpu_time = timer.Stop();
	for(uint n = 1; n<= N; n++){
		cout << "T[" << n << "]= " << T(n) << endl;
	}
	cout << "CPU time = " << cpu_time << "sec." << endl;
	Wait();
}

void TanCoeffTest(uint N)
{
	SNBlock <SLong> T(N+1);
	Timer timer;
	double cpu_time;
	uint n;

	cout << "Calcuating tangent coefficients by TanCoeff() function." << endl;
	timer.Start();
	for(n = 1; n <= N; n++) T[n-1] = TanCoeff(n);
	cpu_time = timer.Stop();

	for(n = 1; n <= N; n++){
		cout << "T[" << n << "]=" << endl;
		cout << T(n-1) << endl;
	}
	cout << "CPU time = " << cpu_time << " sec." << endl;
	Wait();
}

void BnTableTest(uint N)
{
	SNBlock <SFraction> B(N+1);
	Timer timer;
	double cpu_time;

	cout << "Calculating Bernouilli numbers by BernNumTable() function." << endl;
	timer.Start();
	BernNumTable(B, N);
	cpu_time = timer.Stop();
	for(uint n = 1; n<= N; n++) {
		cout << "B[" << n << "]= " << B[n] << endl;
	}
	cout << "CPU time = " << cpu_time << " sec." << endl;
	Wait();
/*
    ofstream outF;
    outF.open("BnTable.txt", ios::out);
    if( !outF ) {
        cerr << "Cannot open output stream." << endl;
        return;
    }
	for(uint n = 1; n<= N; n++) {
		outF << "B[" << n << "]= " << B[n] << endl;
	}
	outF.close();
	Wait("BnTable.txt has been made.");
*/
}

void BernNumTest(uint N){
	SNBlock <SFraction> B(N+1);
	Timer timer;
	double cpu_time;
	uint n;

	puts("Calculating Bernouilli numbers by BernNum() function with combL().");
	timer.Start();
	for(n = 1; n <= N; n++) B[n] = BernNum(n);
	cpu_time = timer.Stop();
	for(n = 1; n <= N; n++){
		cout << "B[" << n << "]= " << B[n] << endl;
	}
	cout << "CPU time = " << cpu_time << " sec." << endl;
	Wait();
}

void EulerNumTest(uint N){
	SNBlock <SLong> E(N+1);
	Timer timer;
	double cpu_time;
	uint n;

	cout << "Calculating Euler numbers by EulerNum() function." << endl;
	timer.Start();
	for(n=0; n<= N; n++) E[n] = EulerNum(n);
	cpu_time = timer.Stop();

	for(n = 0; n<= N; n++){
		cout << "E[" << n << "]= " << E(n) << endl;
	}
	cout << "CPU time = " << cpu_time << " sec." << endl;
	Wait();
}

int main()
{
	uint N;
	SFraction F;
	F.SetFormat( F.BRACKET | F.END_CR);

	cerr << title << endl;
	cerr << "How many terms do you want? ";
	N = GetInt();

	TnTableTest(N);
	TanCoeffTest(N);
	BnTableTest(N);
	BernNumTest(N);
	EulerNumTest(N);

	Ending();
}
