/*  smpcmpx3.cpp  by K.Tsuru  */
/*
This program compares SComplex class with C++ complex one.
It has been tested on gcc.
*/
#include "sn.h"
#include "wait.h"
typedef complex<double> dcomplex;
static const char* const title = "A test program for SComplex class";

void ShowTitle(const char* func){
	cout << "****************** test of " << func << " ********************" << endl;
}
void TestCpowD(){
	ShowTitle("Cpow(z, p) the power of real");
	SComplex zs, rs, diff;
	dcomplex zd, rd;  // About "dcomplex" see above.
	SDouble xs, pi = Pi();
	const SDouble p = Sqrt(2.0);
	double xd;
	const double cf = 2.5;
	int i, N = 8;
	cout << "q = z^p with p =" << p << endl;
	cout<< "p/pi   |   SComplex value   |  complex <double> value  |  difference from double" << endl;
	for(i = -N; i<= N;i++){
		if(i == 0) continue;	// If z == 0, an error causes.
		xd = 2*i/(double)N; // double
		xs = xd*pi;  // SD
		zs = SComplex(xs, cf*xs);
		zd = complexD(zs); // dcomplex(doubleD(x), doubleD(cf*x));
		rd = pow(zd, doubleD(p) ); // zd^p
		rs = Cpow(zs, p);
		diff = rd - rs;
		cout << zd <<"  "<< rd << "  "<< complexD(diff) << endl;
	}
}
void TestCpowC(){
	ShowTitle("Cpow(z, p) the power of complex");
	SComplex z0, zp0, zs, zsp, zsr, diff;
	const SDouble pi = Pi(), c = Sqrt(2.0);
	const int N = 10;
	int i;
	dcomplex zd, zdp, zdr;

	z0.Set(pi, 2.5*c);	z0 /= N;
	zp0.Set(c, 0.3*pi);  zp0 /= N;

	cout << " z   |   p     |    complex <double> value  |  difference from double" << endl;
	for(i=-N; i<= N;i++){
		if(i == 0) continue;
		zs  = i * z0;
		zsp = i * zp0;
		zsr = Cpow(zs, zsp);
		zd  = complexD(zs);
		zdp = complexD(zsp);
		zdr  = pow(zd, zdp);
		diff = zsr - zdr;
		cout << zd << " " << zdp << " " << zdr << " " << complexD(diff) << endl;
	}
}
int main(){
	RealSize C;
	C.SetEffFig(30);
	SDouble::SetFormat(10,10);
	cerr << title << endl;
	TestCpowC();
	Wait();
	TestCpowD();
	Ending();
}
