/*  SMPTRG2.CPP  by K.Tsuru  */ 
static const char* title = "<<<<< A sample program for trigonometric functions >>>>>\n";

#include "sn.h"

void ShowCalcInfo(const SDouble& c, double cpu_time){
	printf("Up to %ldth terms, cpu time = %.3f\n",c.UpToTerm(), cpu_time);
}
void DidYouGet(double x){
	fprintf(stderr, "Did you get a value very close to %g ?\n", x);
}
void CosTest(){
	SDouble  pi, a, b, c;
	Timer timer;
	double cpu_time;

	pi = Pi();
	puts("cos(0) =");
	b = Cos(0.0);
	b.Puts();
	printf("Up to %ldth terms\n",b.UpToTerm());

	puts("Cos(1) =");
	timer.Start();
	c = Cos(1.0);
	cpu_time = timer.Stop();
	c.Puts();
	ShowCalcInfo(c, cpu_time);
	Wait();

	puts("b = Cos(pi/6) =");
	timer.Start();
	b = Cos(pi/6);
	cpu_time = timer.Stop();
	b.Puts();
	ShowCalcInfo(c, cpu_time);

	puts("Squre of b.");
	a = b*b;
	a.Puts();
	DidYouGet(0.75);
	Wait();

	puts("b = Cos(pi/4) =");
	timer.Start();
	b = Cos(pi/4);
	cpu_time = timer.Stop();
	b.Puts();
	ShowCalcInfo(c, cpu_time);
	puts("Squre of b.");
	a = b*b;
	a.Puts();
	DidYouGet(0.5);
	Wait();

	puts("Cos(pi/2) =");
	timer.Start();
	b = Cos(pi/2);
	cpu_time = timer.Stop();
	b.Puts();
	DidYouGet(0.0);
	ShowCalcInfo(c, cpu_time);
	Wait();
}

void SinTest(){
	SDouble  pi,a,b, c;
	Timer timer;
	double cpu_time;

	pi = Pi();
	puts("Sin(0) =");
	a = 0.0;
	b = Sin(a);
	b.Puts();

	puts("Sin(1) =");
	timer.Start();
	c = Sin(1.0);
	cpu_time = timer.Stop();
	c.Puts();
	ShowCalcInfo(c, cpu_time);
	Wait();

	puts("Sin(pi/6) =");
	timer.Start();
	b=Sin(pi/6);
	cpu_time = timer.Stop();
	b.Puts();
	DidYouGet(0.5);
	ShowCalcInfo(c, cpu_time);
	Wait();

	puts("b = cos(pi/4) =");
	timer.Start();
	b = Sin(pi/4);
	cpu_time = timer.Stop();
	b.Puts();
	ShowCalcInfo(c, cpu_time);
	puts("Squre of b.");
	a=b*b;
	a.Puts();
	DidYouGet(0.5);
	Wait();

	puts("sin(pi/2) =");
	b=Sin(pi/2);
	b.Puts();
	DidYouGet(1.0);
	Wait();
}
void AsinTest(){
	SDouble  pi,a,b, c;
	Timer timer;
	double cpu_time;

	pi = Pi();
	puts("Calculating Asin(0.5)");
	timer.Start();
	a = Asin(0.5);
	cpu_time = timer.Stop();
	b = pi - 6*a;
	puts("pi - 6*Asin(0.5) =");
	b.Puts();
	DidYouGet(0.0);
	ShowCalcInfo(c, cpu_time);
	Wait();

	puts("Calculating Asin(1/Sqrt(2))");
	timer.Start();
	c = RecSqrt(2);
	a = Asin(c);
	cpu_time = timer.Stop();
	b = pi - 4*a;
	puts("pi() - 4*Asin(1/Sqrt(2)) =");
	b.Puts();
	DidYouGet(0.0);
	ShowCalcInfo(c, cpu_time);
	Wait();
}
SDouble AtanByAsin(const SDouble& x){
	SDouble y = x*RecSqrt(1 + x*x);
	return Asin(y);
}
void AtanTest(){
	SDouble  pi, a = Sqrt(3), d;
	Timer timer;
	double cpu_time;

	puts("Calculating pi = 4*Atan(1).");
	timer.Start();
	pi = 4*Atan(1);
	cpu_time = timer.Stop();
	pi.Puts(10, 0, 5, 49);
	ShowCalcInfo(pi, cpu_time);
	puts("Pi() - 4*Atan(1) =");
	d = Pi() - pi;
	d.Puts();
	DidYouGet(0.0);
	Wait();

	puts("Calculating 4*Atan(1) by using Asin(1).");
	timer.Start();
	pi = 4*AtanByAsin(1);
	cpu_time = timer.Stop();
	pi.Puts(10, 0, 5, 49);
	ShowCalcInfo(pi, cpu_time);
	puts("Pi() - 4*Asin( x/Sqrt(1 + x*x) ) =");
	d = Pi() - pi;
	d.Puts();
	DidYouGet(0.0);
	Wait();

	puts("Calculating 3*Atan(Sqrt(3.0)).");
	timer.Start();
	pi = 3*Atan(a);
	cpu_time = timer.Stop();
	pi.Puts(10, 0, 5, 49);
	ShowCalcInfo(pi, cpu_time);
	puts("Pi() - 3*Atan( Sqrt(3.0) ) =");
	d = Pi() - pi;
	d.Puts();
	DidYouGet(0.0);
	Wait();
}
void SquareSumTest(){
	SDouble  a, b, c;
	double x;

	puts("Test Cos(x)^2 + Sin(x)^2 = 1.0 ?");
	while(1){
		fprintf(stderr,"x = ? (abort when x = 0) ");
		x = GetDouble();
		if(x == 0.0) break;
		a = x;
		puts("Your x =");
		a.Puts();

		puts("Calculating Sin(x) and Cos(x)");
		b = Sin(a);
		c = Cos(a);

		puts("Calculating Cos(x)^2 + Sin(x)^2 - 1.0");
		a = b*b + c*c - 1.0;
		a.Puts();
		DidYouGet(0.0);
		Wait();
	}
}
int main(){
	RealSize C;
	uint fig;
	ErrorPuts(title);
	ErrorPut("Effective figures/4 = ? ");
	fig = GetInt();
	C.SetEffFig(fig);
	SDouble  pi;
	long p;

	puts("pi =");
	pi = Pi();
	p = pi.Puts(10, 0, 5, 49);
	printf("Number of output characters = %ld.\n", p);
	Wait();

	AtanTest();
	AsinTest();
	SinTest();
	CosTest();
	SquareSumTest();

	Ending();
	return EXIT_SUCCESS;
}
