/*  ChudPiSpeedTest.cpp  by K.Tsuru  */
#include "sn.h"

/*
Speed test of pi by Chudnovskys' formula.
132,000,000 digits ... out of memory... FFT_MAX_BITS = 23
100,000,000... out of memory... FFT_MAX_BITS = 22

 33,550,000 digits 156.786(sec) 2.6min... FFT_MAX_BITS = 23
				   239.843(sec) 4.0min... FFT_MAX_BITS = 22
 50,000,000        365.903      6.1min
2017/03/22 version=========================
 50,000,000        301.794      5.03min
 10,000,000         52.02       0.867min

2017/03/30 64bit version=========================
 10,000,000          91.4252(sec) [1.52375 min]10000029(figures) slow?
130,000,000 digits 1681(sec) [28.0167 min] 130000029(figures) memory needs about 2GB

2017/04/01 64bit version=========================
KaratsubaMult() is  used.
How many digits do you want? = 1,0000,0000
****************** Speed test of pi by Chudnovskys' formula. ********************
****************** ChudnovskysPi ********************
Now computing in 100000000 digits.
ChudnovskysPi CPU time = 1096(sec) [18.2667 min]100000029 (figures)
=============================================================
KaratsubaMult() is  used.
How many digits do you want? = 2 0000 0000
****************** Speed test of pi by Chudnovskys' formula. ********************
****************** ChudnovskysPi ********************
Now computing in 200000000 digits.
ChudnovskysPi CPU time = 2795(sec) [46.5833 min]200000029 (figures)
KaratsubaMult() is not used.
How many digits do you want? = 200000000
****************** Speed test of pi by Chudnovskys' formula. ********************
****************** ChudnovskysPi ********************
Now computing in 200000000 digits.
ChudnovskysPi CPU time = 3617(sec) [60.2833 min]200000029 (figures)
Now computing in 400000000 digits.
ChudnovskysPi CPU time = 7455(sec) [124.25 min]400000029 (figures)

Now computing in 600005000 digits.
ChudnovskysPi CPU time = 14168(sec) [236.133 min]600005029 (figures)=3.93555h = about 4hours
The result agrees with the "Pi Digits" by F.Bellard.
URL--> http://bellard.org/pi/pi2700e9/pidigits.html

****************** ChudnovskysPi ****2017/08/07********MinGW64********
Now computing in 100000000 digits. Start at 15:27:20
fig = 25000000
ChudnovskysPi CPU time = 1013(sec) [16.8833 min]100000029 (figures)
****************** ChudnovskysPi ****2017/09/17********MinGW64********
FFTSinTable is used.
in MakeSinTable() t = 0.0807963
ChudnovskysPi CPU time = 1103(sec) [18.3833 min]100000029 (figures) ... nearly the same
****************** ChudnovskysPi ****2017/10/16********MinGW64*********
Now computing in 1000000000 digits. Start at 16:53:21
ChudnovskysPi CPU time = 35988(sec) [599.8 min]1000000029 (figures)
Ten hours.

How many digits do you want? = 10,000,000
****************** Speed test of pi by Chudnovskys' formula. ********************
****************** ChudnovskysPi ********************
Now computing in 10000000 digits. Start at 11:17:59
ChudnovskysPi CPU time = 73.9031(sec) [1.23172 min]10000029 (figures)
===Uses FFTSineTable ========================================
Now computing in 10000000 digits. Start at 14:52:44
in MakeSinTable() t = 0.0781126
ChudnovskysPi CPU time = 73.2936(sec) [1.22156 min]10000029 (figures) almost the same speed
==================================*/

void ShowTitle(const char* title){
	cout << "****************** " << title << " ********************" << endl;
}
void FileOut(const SDouble& data, const string& fname, const string& title) {
	ofstream fout(fname.c_str(),ios::trunc);	 // open file

	data.SetFormat(10, 0, 10, data.INT_PUT|data.CRLF);
//	data.SetFormat(10, 0, 5, data.INT_PUT|data.CRLF|data.SHOW_FIG); // add the number of figures at line end
	fout << title << " (" << data.DFigures() << " digits)" << endl;
	fout << data << endl << " (" << data.DFigures() << " digits)" << endl;
	fout.close();
}
SDouble ConstFuncSpeed(SDouble (*sdfunc)(), const char* fname) {
	Timer timer;
	SDouble r = 0.0;
	ShowTitle(fname);
	cerr << "Now computing in " << 4*r.EffFig() << " digits. Start at " << WhatTimeNow() << endl;
	timer.Start();
	r =ChudnovskysPi();
	double time = timer.Stop();

	cout << fname << " CPU time = " << time << "(sec) [" << (time/60.0) << " min]" << r.DFigures() << " (figures)"<< endl;

	return r;
}
typedef long long llint;
#if defined _WIN64
#define OS 64
#else
#define OS 32
#endif

int main() {
	RealSize C;
	SLong t;
//	t.KaratsubaMultSwitch(ON);
	string msg = "KaratsubaMult() is ";
	msg += t.UsesKaratsubaMult() ? "" : "not";
	msg += " used.";
	cout << msg << endl;
	cerr << "How many digits do you want? = ";
	int fig = GetInt();
	C.SetEffFig(fig/4);
//cout << Pi() << endl;
	ShowTitle("Speed test of pi by Chudnovskys' formula.");
	SDouble piCh = ConstFuncSpeed(ChudnovskysPi, "ChudnovskysPi");
	cout << "===========================================" << endl;
//	FFTInfo();
	SNInfo();
	cerr << "Do you output to file ?(Y/N)";
	if(YesOrNo()) {
		FileOut(piCh, "ChudnovskysPi.dat", "Pi =");
		cerr << "Output" << " a file : ChudnovskysPi.dat " << endl;
	}	
	Ending();
}
