/* Compare_SD_dbl_test.cpp by K.Tsuru*/
/*
It compares SDouble functions with double ones.
*/

#include "sn.h"
#include "timer.h"
#include "getnum.h"
#include "Wait.h"
static const char* const title = "******** A test program for SDouble functions ********";

void ShowTitle(const char* func){
	cout << "****************** test of "<< func << " ********************" << endl;
}
bool isSmall(const SDouble& x) {
	static const SDouble err = 1.0e-11;
	return ( Dabs(x) < err ); 
}

void XFunc_SD_dbl_Test(SDouble (*pSDfunc)(const SDouble&), double absmax,
    double (*pdfunc)(const double), const char* fname, bool positive=false, double min = 0.0)
{
	Timer timer;
	ShowTitle(fname);
	int c = 0;

	SDouble rs, diff;
	double rd, lap = -1.0;
	double a = sqrt(2.0);
	cout << "  x(double)    f(x)(double)          Relative error from double" << endl;
	rs.SetFormat(0, 7);
	cout.precision(7);	
	const int N = 10;
	int i, Nmin = positive ? 1 : -N;
	double x, max = abs(absmax);
	while(N*a > max) a /= 2;
	if(min > 0) while(a*Nmin < min) a *= 2;
	timer.Start();
	for(i = Nmin; i <= N; i++) {
		if(!i){	lap =timer.LapTime();  continue; }
		x = i * a;
		rs = (*pSDfunc)(x);
		rd = (*pdfunc)(x);
		c++;
		diff = (rd != 0.0) ? (1.0 - rs/rd) : 0.0;
		cout.width(10);	cout.fill(' ');
		cout << setprecision(5) << fixed << x <<"  " <<  rd << "     "<< scientific << diff << endl;
		if(!isSmall(diff)){
			cerr << "x = " << x << "  rs = " << rs << "  rd = " << rd <<endl;
			cerr << "error is large" << endl; Wait();
		}
	}
	double time = timer.Stop();
	cout.precision(5);
	cout << "-----------------------" << endl;	
	cout << fname << " CPU time = " << time << "(sec)  " << (time/c) << "sec/once";
	if(lap > 0) cout << "  half lap time = "<< lap;
	cout << endl<< "in effective figures = " << rs.EffFig() << endl;
	Wait();
}
/*
ExpSeries(x);
CPU time is the same for x>0 and x<0
*/
void SpeedTestExpSeries(){
	cout << "ExpSeries() test  x     y" << endl;
	Timer timer;

	SDouble x, y, a = 0.01*Sqrt(0.5);
	double time, lap = -1.0;
	int i, c = 0;
	cout.precision(5);
	timer.Start();
	for(i = -10; i <= 10; i++) {
		if(!i){	lap =timer.LapTime();  continue; }
		x = i * a;
		y = ExpSeries(x);
		cout << "i = " << i << "  " << doubleD(x) << "  " << doubleD(y) << endl; 
		c++;
	}
	time = timer.Stop();
	cout.precision(5);
	cout << "-----------------------" << endl;	
	cout << " CPU time = " << time << "(sec)  " << (time/c) << "sec/once";
	if(lap > 0) cout << "  half lap time = "<< lap;
	cout << endl<< "in effective figures = " << x.EffFig() << endl;
}

void TestExpMultEq1(SDouble (*expfunc)(const SDouble&), const char* fname){
	cout << "Test of exp(x) function "<<fname<<"(x)*"<<fname<<"(-x)=1.0 or not" << endl;
	SDouble x, c = Sqrt(5.6), a,b, z;

	int j;
	for(j= 1; j<= 10; j++) {
		x = j*c;
		a = (*expfunc)(x);
		b = (*expfunc)(-x);
		z = a*b;
		cout << j <<" : " << z <<  endl;
	}
	cout << "All values are 1.0?" << endl;

	Wait();
	
}
void DpowSpeedTest(){
	SDouble exp1= E(), rexp1 = RecE();
	SDouble yp;
	double time;
	Timer timer;
	int j, N=1000;

	timer.Start();
	for(j=1;j<= N;j++) yp = Dpow(exp1,j);
	time = timer.Stop();
	cout << "For e^n 0 time = " << time << endl;

	timer.Start();
	for(j= 1;j<= N;j++) yp = Dpow(rexp1, j);
	time = timer.Stop();
	cout << "For (1/e)^n time = " << time << endl;

	Wait();	
	
}
void E_RecE_SpeedTest(){
	cout << "E_RecE_SpeedTest()"<< endl;
	SDouble yp;
	double time;
	Timer timer;
	int j, N=10;

	timer.Start();
	for(j=0;j<N;j++) yp = E();
	time = timer.Stop();
	cout << "E() time = " << time << endl;

	timer.Start();
	for(j= 0;j< N;j++) yp = RecE();
	time = timer.Stop();
	cout << "RecE() time = " << time << endl;

	Wait();	

}
void ExpowerSpeedTest(){
	cout << "ExpowerSpeedTest()"<< endl;
	SDouble yp;
	double time;
	Timer timer;
	int j, N=100;

	yp = E(); // to prepare constant e
	timer.Start();
	for(j=0;j<N;j++) yp = Expower(j);
	time = timer.Stop();
	cout << "e^j (j > 0) time = " << time <<"  " << N << " times"<< endl;

	timer.Start();
	for(j= 0;j< N;j++) yp = Expower(-j);
	time = timer.Stop();
	cout << "e^j (j< 0) time = " << time <<"  " <<  N << " times" << endl;

	Wait();	
	
}
void ExpDS_SpeedTest(){
	cout << "ExpDS_SpeedTest()"<< endl;
	SDouble x,yp,ym, c = Sqrt(5.6);
	double time;
	Timer timer;
	int j;
	yp = ExpDS(c);
	timer.Start();
	for(j=0;j<10;j++){
		x = (j+1)*c;
		yp = ExpDS(x);
	}
	time = timer.Stop();
	cout << "For x > 0 time = " << time << endl;

	timer.Start();
	for(j=0;j<10;j++){
		x = -(j+1)*c;
		ym = ExpDS(x);
	}
	time = timer.Stop();
	cout << "For x < 0 time = " << time << endl;
	Wait();	
}
int main(){
	RealSize C;
	ErrorPuts(title);
	double mtx = 1.0, mx = 8.0, pi4 = M_PI/4.0;

	C.SetEffFig(2000);
	SNManager::ShowMessage(1);
	ExpowerSpeedTest();

	E_RecE_SpeedTest();

	C.SetEffFig(0);
	C.SetEffFig(1000);
	ExpDS_SpeedTest();
	TestExpMultEq1(ExpBS, "ExpBS");
	TestExpMultEq1(ExpDS, "ExpDS");

	XFunc_SD_dbl_Test(ExpDS, mx,exp, "ExpDS");
	XFunc_SD_dbl_Test(ExpBS, mx,exp, "ExpBS");
	XFunc_SD_dbl_Test(Atanh, mtx, atanh, "Atanh");
	XFunc_SD_dbl_Test(LogNW, mx, log, "LogNW", true);
	XFunc_SD_dbl_Test(LogE, mx, log, "Log = LogE", true);
	XFunc_SD_dbl_Test(LogAGM, mtx, log, "LogAGM 0.001 < x < 1.0", true);
	XFunc_SD_dbl_Test(Log10, mx, log10, "Log10", true);


	XFunc_SD_dbl_Test(SinRN, mx, sin,"SinRN");
	XFunc_SD_dbl_Test(SinDiv, pi4, sin, "SinDiv");
	XFunc_SD_dbl_Test(SinS_RN, mx, sin,"SinS_RN");

	XFunc_SD_dbl_Test(CosRN, mx, cos, "CosRN");
	XFunc_SD_dbl_Test(CosDiv, pi4, cos, "CosDiv");
	XFunc_SD_dbl_Test(CosS_RN, mx,cos, "CosS_RN");
	XFunc_SD_dbl_Test(Sin, mx,sin, "Sin");
	XFunc_SD_dbl_Test(Cos, mx,cos, "Cos");
	XFunc_SD_dbl_Test(Tan, mx,tan, "Tan");

	XFunc_SD_dbl_Test(Sinh, mx, sinh, "Sinh");
	XFunc_SD_dbl_Test(Cosh, mx, cosh, "Cosh");
	XFunc_SD_dbl_Test(Tanh, mx, tanh, "Tanh");

	XFunc_SD_dbl_Test(Asin, mtx, asin, "Asin");
	XFunc_SD_dbl_Test(Acos, mtx, acos, "Acos");
	XFunc_SD_dbl_Test(Atan, mx, atan, "Atan");
	XFunc_SD_dbl_Test(AtanA, mx, atan, "AtanA");
	XFunc_SD_dbl_Test(AtanT, mx, atan, "AtanT");
	XFunc_SD_dbl_Test(Asinh, mx, asinh, "Asinh");
	XFunc_SD_dbl_Test(Acosh, mtx, acosh, "Acosh",true,1.0);
	XFunc_SD_dbl_Test(Atanh, mtx, atanh, "Atanh");
	XFunc_SD_dbl_Test(Sqrt, mx,sqrt, "Sqrt", true);
	C.SetEffFig(0);
	Ending();
}
