/* FFTLimitTest.cpp by K. Tsuru */
/*test program for FFT multiplication
in "snconst.h"
const int FFT_MAX_SIZE_BITS = 23;	// 20 --> 22 --> 23(OK) --> 24(NG) ;	// bits of maximum size of FFT work area
in "sfft.h"
const uint fftMaxArraySize = 1u << FFT_MAX_SIZE_BITS; // = 22
long double version
====================================
In radix 10000 it tests the upper limit of FFT multiplication p = 8388608
Test of (10000^8388608-1)^2 + 2*(10000^8388608-1)+1 == 10000^16777216
Checking now.
a = DRADIX^p - 1
b=a*a FFTUsedTimes() = 1 <-- 1?
10000^(16777216) one figure ?
Head() = 16777216, Tail() = 16777216 , c[16777216] = 1
Did you get Head() = 16777216 ?
j = 14, p = 8388608, a.FFTMaxArraySize() = 134217728
====================================
In radix 10000 it tests the upper limit of FFT multiplication p = 16777216
Test of (10000^16777216-1)^2 + 2*(10000^16777216-1)+1 == 10000^33554432
Checking now.
a = DRADIX^p - 1
b=a*a FFTUsedTimes() = 1 <-- 1?
10000^(33554432) one figure ?
Head() = 33554431, Tail() = 33339540 , c[33554431] = 9999 ... not equal 1
Did you get Head() = 33554432 ?
j = 15, p = 16777216, a.FFTMaxArraySize() = 134217728
====================================
*/
#include "sn.h"
void bar(){
	cout << "====================================" << endl;
}
void LRandMultTest(){
	uint sz = 1024,  j;
	Timer timer;
	double t;
	SLong a, b, c;
	cout << "size of a       b       c    time(sec)" << endl;
	for(j = 0; j< 15; j++){
		a = LRand(sz);	b = LRand(sz);
		a.FFTUsedTimes(true); // reset
		timer.Start();
		c = a*b;
		t = timer.Stop();
		cout << a.FigureSize() << "  " << b.FigureSize() << "   " << c.FigureSize() << "    "<< t << endl;
		cout << "c=a*b FFTUsedTimes() = " << a.FFTUsedTimes() << endl;
		sz *=2;
	}
	cout << "a.FFTMaxArraySize() = " << a.FFTMaxArraySize() << endl;
	FFTInfo();	Wait();
}
int DFFT_limit_test(uint p, bool wait = true){
	SLong a(1), b, c;
	uint h, t;
	printf("In radix %u it tests the upper limit of FFT multiplication p = %u \n", DRADIX, p);
	fprintf(stderr,"Test of (%u^%u-1)^2 + 2*(%u^%u-1)+1 == %u^%u\n", DRADIX, p,DRADIX, p,DRADIX,2u*p);
	ErrorPuts("Checking now.");
//cout << " p = " << p << endl;
	a.MultPowRdx(p);// a = DRADIX^p = 1 0000 0000 0000 ... 0000
	a.FFTUsedTimes(true); // reset
	cout << "a = DRADIX^p - 1" << endl;
	a--;			// a - 1= 9999 9999 9999 ... 9999

	b=a*a;			// fft

	int usedTime = a.FFTUsedTimes();
	cout << "b=a*a FFTUsedTimes() = " << usedTime << " <-- 1?" << endl;

	c = b + 2*a +1; // c = 1 0000 0000 0000 ... 0000
//cout << c<< endl;
	printf("%u^(%u) one figure ?\n",DRADIX, 2*p);
	h = c.Head();	t = c.Tail();
	printf("Head() = %u, Tail() = %u , c[%u] = %u <--- 1?\n", h, t, h, c(t));
	printf("Did you get Head() = %u ?\n",2u*p);
	if(wait) Wait();
	return usedTime;
}

int BFFT_limit_test(uint p, bool wait = true){
	SInteger a(1), b, c;
	uint h, t;
	printf("In radix %u it tests the upper limit of FFT multiplication p = %u \n", BRADIX, p);
	fprintf(stderr,"Test of (%u^%u-1)^2 + 2*(%u^%u-1)+1 == %u^%u\n", BRADIX, p,BRADIX, p,BRADIX,2u*p);
	ErrorPuts("Checking now.");

	a.MultPowRdx(p);// a = BRADIX^p = 1 0000 0000 0000 ... 0000
	cout << "a = BRADIX^p" << endl;
	a.FFTUsedTimes(true); // reset
//cout << "a  = "; a.RawPuts();
	a--;			// a - 1= 32767 32767 32767 ... 32767
//cout << "a  = "; a.RawPuts();
	b=a*a;			// fft
//cout << "a*a= "; b.RawPuts();
	int usedTime = a.FFTUsedTimes();
	cout << "b=a*a FFTUsedTimes() = " << usedTime << " <-- 1?" << endl;

	c = b + 2*a +1; // c = 1 0000 0000 0000 ... 0000
//cout << "c = "; c.RawPuts();
	printf("%u^(%u) one figure ?\n",BRADIX, 2*p);
	h = c.Head();	t = c.Tail();
	printf("Head() = %u, Tail() = %u , c[%u] = %u <--- 1?\n", h, t, h, c(t));
	printf("Did you get Head() = %u ?\n",2u*p);
	if(wait) Wait();
	return usedTime;
}
void DFFT_limit_test_varN(uint initVal) {
	cout << "********* DFFT_limit_test_varN() ************" << endl;
	uint p = initVal, j;
	SLong a;
	for(j = 1; j < 25; j++) {
		if(DFFT_limit_test(p, false)>=2) break;
		cout << "j = " << j << ", p = " << p << ", a.FFTMaxArraySize() = " << a.FFTMaxArraySize() << endl;
		p *= 2;
		bar();
    }			
	bar();
	Wait();
}
void BFFT_limit_test_varN(uint initVal) {
	cout << "********* BFFT_limit_test_varN() ************" << endl;
	uint p = initVal, j;
	SInteger a;
	for(j = 1; j < 20; j++) {
		if(BFFT_limit_test(p, false)>=2) break;
		cout << "j = " << j << ", p = " << p << ", a.FFTMaxArraySize() = " << a.FFTMaxArraySize() << endl;
		p *= 2;
		bar();
    }			
	bar();
	Wait();
}
int main(){
//	LRandMultTest();	bar();
	SLong sl;
//	sl.SetFFTMinSize(0); // not use FFT p=65536 OK, use FFT p=512 NG
	SInteger si;
//SNInfo();Wait();

	BFFT_limit_test_varN(8);
//	DFFT_limit_test(100);
//	sl.FFTVerify(true); //Error???
//	BFFT_limit_test(si.FFTMaxArraySize()/4u);
	DFFT_limit_test_varN(1024);
//	DFFT_limit_test(sl.FFTMaxArraySize()/2u);
//	BFFT_limit_test(si.FFTMaxArraySize()/2u);
	SNInfo();
	Ending();
}
