/* FuncComp.cpp by K.Tsuru made on Aug 8, 2016*/
/*
Test program to determine the threshold value of
" const uint thFigXXX = xxxx;"
in snmath.h
Run on your platform and determine them.
For three or over functions please make a tournament selection.
*/

#include "FuncComp.h"
/*
Compare the speeds of two functions.
*/
void XFuncCompareValues(SDouble (*pfunc1)(const SDouble&), const char* fname1,
 SDouble (*pfunc2)(const SDouble&), const char* fname2, const ForStm &F)
{
	RealSize C;
	SDouble x, r1, r2, diff;;

	cout << "Compare values changing effective figures " << fname1 <<" and " << fname2 << endl;
	cout << " ef  |" << " difference" << endl;
	int fig, c = 0;
	for(fig = F.init; fig <= F.max; fig += F.inc){
		C.SetEffFig(fig);
		x = Drand();
		r1.SetFormat(5, 10);
		r1 = (*pfunc1)(x);
		r2 = (*pfunc2)(x);
		diff = r1 -r2;
		cout << fig << "   " << diff<< endl;
//SNInfo();
		C.SetEffFig(0);
		c++;
	}
	cout << c << " times called" << endl;
}

void XFuncSpeedCompare2(SDouble (*pfunc1)(const SDouble&), const char* fname1,
 SDouble (*pfunc2)(const SDouble&), const char* fname2,
  double absmax, const ForStm &F,const int N, bool positive, double min)
{
	Timer timer1, timer2;
	double time1, time2;
	RealSize C;

	SDouble x, r, R = Sqrt(0.5);
x.SetFormat(5,10); 
	SNBlock <SDouble> r1;
	SNBlock <SDouble> r2;
	int fig, c, c1 = 0;
	int i, Nmin = positive ? 1 : -N;
	SDouble max = abs(absmax);
	while(N*R > max) R /= 2;
//	if(min > 0) while(R*Nmin < min) R *= 2;
	cout << "Compare speed depending on effective figures " << fname1 <<" and " << fname2 << endl;
	cout << "            CPU time" << endl;
	cout << " ef  |" << fname1 << "  | " << fname2 << " | ratio  | faster function" << endl;
	for(fig = F.init; fig <= F.max; fig += F.inc){
		C.SetEffFig(fig);
//		R = max;
 		timer1.Start();
 		for(c= 0, i = Nmin; i <= N; i++, c++) {
			if(!i){ c--; continue; }
			x = i * R;
			r1.reserve(c);
//cout << x << endl;
			r1[c] = (*pfunc1)(x);
		}
		time1 = timer1.Stop();
		if(!c1) c1 = c;
		timer2.Start();
		for(c = 0, i = Nmin; i <= N; i++, c++) {
			if(!i){ c--; continue; }
			x = i * R;
			r2.reserve(c);
			r2[c] = (*pfunc2)(x);
		}
		time2 = timer2.Stop();

		cout.precision(5);
		cout << x.EffFig() <<"  " << time1 << "  " << time2 <<"  " << (time1/time2) <<"  " << ((time1 < time2) ? fname1 : fname2) << endl;
		C.SetEffFig(0);
	}
	cout << c1 << " times called for each roop" << endl;
	XFuncCompareValues((*pfunc1), fname1, (*pfunc2), fname2, F);	
}
void XFuncSpeedDependOnFig(SDouble (*pfunc)(const SDouble&), const char* fname,
 double absmax, const ForStm &F,const int N, bool positive, double min)
{
	Timer timer;
	double time;
	RealSize C;

	SDouble x, r, R = Sqrt(0.5);
	int fig, c;
	int i, Nmin = positive ? 1 : -N;
	SDouble max = abs(absmax);
	while(N*R > max) R /= 2;
	if(min > 0) while(R*Nmin < min) R *= 2;
	cout << "Compare speed depending on effective figures " << fname << endl;
	cout << " ef  |" << "  CPU time" << endl;
	for(fig = F.init; fig <= F.max; fig += F.inc){
		C.SetEffFig(fig);
		R = Sqrt(0.5);
 		timer.Start();
 		c = 0;
		for(i = Nmin; i <= N; i++) {
			if(!i) continue;
			x = i * R;
			r = (*pfunc)(x);
			c++;
		}
		time = timer.Stop();

		cout.precision(5);
		cout << x.EffFig() <<"  " << time << " ( " << c << " times called)" << endl;
		C.SetEffFig(0);
	}
}
