/* functest.cpp by K.Tsuru*/
/*
It compares SDouble functions with double ones.
*/

#include "sn.h"
// #include "SNextern.h" // since version 2.20

static const char* const title = "******** A test program for SDouble functions ********";

void ShowTitle(const char* func){
	printf("****************** test of ");
	printf("%s", func);
	puts(" ********************");
	puts("              x               double    test of doubleD   difference");
}
void ShowCalcInfo(const SDouble& c, double cpu_time){
	printf("Up to %ldth terms, cpu time = %.3f\n",c.UpToTerm(), cpu_time);
}
bool isSmall(const SDouble x) {
	static const SDouble err = 1.0e-11;
	return ( Dabs(x) < err ); 
}

void XFuncSpeed(SDouble (*pfunc)(const SDouble&), double absmax,
    double (*dfunc)(const double), const char* fname, bool positive=false, double min = 0.0)
{
	Timer timer;
	int c = 0;
	RealSize C;

	SDouble r, x, R = Sqrt(0.5);
	double halfLap;// , rD;
	uint fig;
	const int N = 10;
	int i, Nmin = positive ? 1 : -N;
	SDouble max = abs(absmax);
	while(N*R > max) R /= 2;
	if(min > 0) while(R*Nmin < min) R *= 2;
	cout << "Speed depending on effective figures of " << fname << endl;
	cout << " effective figures  " << "CPU time" << "  time/once" << "   half lap" << endl;
	for(fig = 1000; fig < 2500; fig += 100){
		c = 0;
		C.SetEffFig(fig);
		R = Sqrt(0.5);
 // rD = doubleD(R); cout << "R = " << rD << endl; 
		timer.Start();
		for(i = Nmin; i <= N; i++) {
			if(!i){	halfLap =timer.LapTime();  continue; }
			x = i * R;
			r = (*pfunc)(x);
			c++;
		}
		double time = timer.Stop();
		cout.precision(5);	
		cout << r.EffFig() <<"  " << time << "  " << (time/c) <<"  " <<  halfLap << endl;
		C.SetEffFig(0);
	}
}

void XFuncTest(SDouble (*pfunc)(const SDouble&), double absmax,
    double (*dfunc)(const double), const char* fname, bool positive=false, double min = 0.0)
{
	Timer timer;
	ShowTitle(fname);
	int c = 0;

	SDouble rs, diff;
	double rd, lap;
	double a = sqrt(2.0);
	cout << "  x     f(x) value       difference from double" << endl;
	rs.SetFormat(0, 7);
	cout.precision(7);	
	const int N = 10;
	int i, Nmin = positive ? 1 : -N;
	double x, max = abs(absmax);
	while(N*a > max) a /= 2;
	if(min > 0) while(a*Nmin < min) a *= 2;
	timer.Start();
	for(i = Nmin; i <= N; i++) {
		if(!i){	lap =timer.LapTime();  continue; }
		x = i * a;
		rs = (*pfunc)(x);
		rd = (*dfunc)(x);
		c++;
		diff = rd - rs;
		cout << x <<"  "<< rd << "     "<< diff << endl;
		if(!isSmall(diff)){
			cerr << "x = " << x << "  rs = " << rs << "  rd = " << rd <<endl;
			cerr << "error is large" << endl; Wait();
		}
	}
	double time = timer.Stop();
	cout.precision(5);	
	cout << fname << " CPU time = " << time << "(sec)  " << (time/c) << "sec/once  lap time = "<<lap << endl;
	cout << "in effective figures = " << rs.EffFig() << endl;
Wait();
}

void logTest(SDouble (*pfunc)(const SDouble&)) {
	SDouble c = 3.0;
	SDouble x = Sqrt(c), y = (*pfunc)(x), a; // y = log(sqrt(c))
	a = Exp(y); // a = sqrt(c)
	cout << a*a - c << "  = 0.0?" << endl;
	Wait();
}
int main(){
	RealSize C;
	C.SetEffFig(1000); // fig = 10,000
	ErrorPuts(title);
//	logTest(Log);

	double mtx = 1.0, mx = 8.0;//, pi4 = M_PI/4.0;
/*
XFuncTest(Atanh, mtx, atanh, "Atanh");
									   // 1000  | 2,500  5,000  10,000(fig)
	XFuncTest(LogNW, mx, log, "LogNW", true); // 0.166 | 0.260 | 0.4413 | 0.871
	XFuncTest(LogE, mx, log, "Log = LogE", true); // 0.0181 | 0.0997 | 0.296 | 0.588 ... fastest
	XFuncTest(LogAGM, mtx, log, "LogAGM 0.001 < x < 1.0", true); // 0.209 | 0.315 | 0.527 | 1.056
	XFuncTest(Log10, mx, log10, "Log10", true); // 0.0188 |0.1005 | 0.2804 | 0.570

//cout << "Exp(" << x << ")= " << y << endl;
	XFuncTest(ExpBS, mx,exp, "ExpBS"); // 0.165 | 0.181 0.208   0.265 sec
//	XFuncTest(ExpL, mx,exp, "ExpL");   // ......| ..............0.695 sec
	XFuncTest(ExpDS, mx,exp, "ExpDS"); // 0.107 | 0.152 0.288   0.810 
*/
//	XFuncSpeed(Exp, mx,exp, "Exp");
//	XFuncSpeed(ExpBS, mx,exp, "ExpBS");
//	XFuncSpeed(ExpDS, mx,exp, "ExpDS");
	Wait();
//	C.SetEffFig(0);

//	C.SetEffFig(8);
/*
	XFuncTest(SinRN, mx, sin,"SinRN");
	XFuncTest(SinDiv, pi4, sin, "SinDiv");
	XFuncTest(SinS_RN, mx, sin,"SinS_RN");

	XFuncTest(CosRN, mx, cos, "CosRN");
	XFuncTest(CosDiv, pi4, cos, "CosDiv");
	XFuncTest(CosS_RN, mx,cos, "CosS_RN");
*/ 
									   // 1000  | 5,000  10,000(fig)
	XFuncTest(Sin, mx,sin, "Sin");	// 	0.1315
	XFuncTest(Cos, mx,cos, "Cos");	//  0.12027
	XFuncTest(Tan, mx,tan, "Tan");	//  0.11973



	XFuncTest(Sinh, mx, sinh, "Sinh");
	XFuncTest(Cosh, mx, cosh, "Cosh");
	XFuncTest(Tanh, mx, tanh, "Tanh");
	XFuncTest(Asin, mtx, asin, "Asin");
	XFuncTest(Acos, mtx, acos, "Acos");
	XFuncTest(Atan, mx, atan, "Atan");
	XFuncTest(AtanA, mx, atan, "AtanA");
	XFuncTest(AtanT, mx, atan, "AtanT");
	XFuncTest(Asinh, mx, asinh, "Asinh");
	XFuncTest(Acosh, mtx, acosh, "Acosh",true,1.0);
	XFuncTest(Atanh, mtx, atanh, "Atanh");
	XFuncTest(Sqrt, mx,sqrt, "Sqrt", true);
	C.SetEffFig(0);
	Ending();
}
