/*  KaratsubaMultTest.cpp  by K.Tsuru  */
/****************
Karatsuba's multiplication Test
KaratsubaMultSwitch()
off 32.68(sec)
on  16.2(sec) ... two times faster
*****************/
#include "sn.h"
const char *title ="*** The test of Karatsuba's multiplication. ***";

double test1(bool KMswitch, void (*routine)(ulong sz)){
	Timer timer;
	double t, totalTime = 0.0;
	SLong a, b, c;
	ulong size= 1024, maxSize = 131000000ul/4; // change the denominator such as 2, 4, 8...
	a.KaratsubaMultSwitch(KMswitch);
	string OnOff = a.UsesKaratsubaMult() ? "ON": "OFF";
	cout << "KaratsubaMultSwitch() = " << OnOff << endl;
	a.KHHMultUsedTimes(true);
	a.FFTUsedTimes(true);  // reset
	do {
		timer.Start();
		(*routine)(size);
		cout << "size = " << size << ",  FFTUsedTimes() = " << a.FFTUsedTimes(true) << endl;
		if(KMswitch) cout << "Karatsuba's multiplication used times = " << a.KHHMultUsedTimes(true) << "." << endl;
		size *= 2;
		t = timer.Stop();	totalTime += t;
		cout << "CPU time = " << t << "(sec)" << endl;
	}while(size < maxSize);
	return totalTime;
}
void func1(ulong size){
	static bool called = false;
	if(!called) cout << "Now measureting the CPU time of two same size random numbers multiplication." << endl;
	called = true;
	SLong a, b, c;
	a.KHHMultUsedTimes(true);
	a.FFTUsedTimes(true);  // reset
	a = LRand(size);	b = LRand(size);
	c = a*b;
}
void func2(ulong size){
	static bool called = false;
	if(!called) cout << "Now measureting the CPU time of Sqrt() of a random real number." << endl;
	RealSize C;
	C.SetEffFig(size);
	called = true;
	SDouble a, b, r;
	a.KHHMultUsedTimes(true);
	a.FFTUsedTimes(true);  // reset

	r = Drand();
	a = Sqrt(r); //	b = r - a*a;
}
int main(){
	cerr << title << endl << endl;
	double t1, t2;
	cout.precision(5);
	t1 = test1(true, func1);	cout << endl;		t2 = test1(false, func1);
	cout << "Total CPU time " << t1 << "(sec) using Karatsuba multiplication vs. " << t2 << "(sec) not used." << endl;;
	cout << "Speed ratio = " << t2/ t1 << endl;
	Wait();
	t1 = test1(true, func2);	cout << endl;		t2 = test1(false, func2);
	cout << "Total CPU time " << t1 << "(sec) using Karatsuba multiplication vs. " << t2 << "(sec) not used." << endl;;
	cout << "Speed ratio = " << t2/ t1 << endl;
	Wait();
	SNInfo();//	FFTInfo(); is included in SNInfo();
}

