/*  SMPLOGX.CPP  by K.Tsuru  */
const char* title =
"<<<< A sample program for Log(x), Log10() and DpowD(x, p) functions. >>>>\n";

#include "sn.h"
#include "timer.h"
#include "wait.h"
#include "getnum.h"

void ShowCalcInfo(const SDouble& c, double cpu_time){
	cout << "Up to " << c.UpToTerm() << " th terms, cpu time = " << fixed << cpu_time << ", radix = "  << c.Radix() << endl;
}
void Exp1ByDpowD(){
	SDouble X;
	cout << "Compare e with (1+1/x)^x." << endl;

	while(1){
		cerr << "Input a large value x = ? ";
		X = GetString();
		if(X.Sign() <= 0){
			ErrorPuts("x must be positive!");
			continue;
		} else break;
	}

	RealSize C;
	uint fig = (uint)(2*X.RdxExp()) + C.EffFigures();
	C.SetEffFig(fig);

	SDouble one(1.0), x(X), y, z, e = E();
	y = one + one/x;	// y = 1 + 1/x
	cout << "x= " << x << endl;
	cout <<"1+1/x = " << y << endl;
	z = DpowD(y, x);	// z = (1 + 1/x)^x
	cout << "e= " << e << endl;
	cout << " (1+1/x)^x = " << z << endl;
	cout << AgreeUpTo(e, z) << " figures agree. Effective figures =" << z.DFigures() << endl;
	Wait();
	C.SetEffFig(0);
}
void logXSpeedTest(){
	RealSize C;
	Timer timer;
	double cpu_time;
	uint fig;
	cerr << "A test for Log(x). Effective figures / " << DFIGURES << " = ? ";
	fig = GetInt();
	C.SetEffFig(fig);
	SDouble x, ln;

	cout << "Preparing E() in your effective figures." << endl;
	E();
	while(1){
		cerr << "Input x = (Abort when x <= 0)? ";
		x = GetDouble();
		if(x <= 0.0) break;
		cout << "Your x = " << x << endl;
		timer.Start();
		ln = Log(x);
		cpu_time = timer.Stop();
		ln.SetFormat(10, 0, 5, 49);
		cout << ln << endl;
		ShowCalcInfo(ln, cpu_time);
		Wait();
	}
	C.SetEffFig(0);
}
void LogExpTest(){
	Timer timer;
	RealSize C;
	uint fig;

	cerr << "A test program for Log(x) and Exp(x)." << endl;
	cerr << "In the following results hidden figures are shown." << endl;
	fprintf(stderr, "Effective figures / %d = ? ", DFIGURES);
	fig	= abs(GetInt());
	C.SetEffFig(fig);

	SDouble  x, y, z, ln, ln2, d, e;
	double cpu_time;

	puts("********************* Calculating E() ***************************");
	timer.Start();
	e = E();
	cpu_time = timer.Stop();
	e.Puts(5, 0, 10, 49);
	ShowCalcInfo(e, cpu_time);
	Wait();

	puts("******************** Calculating Log(e) ************************");
	timer.Start();
	ln = Log(e);
	cpu_time = timer.Stop();
	ln.Puts(5, 0, 10, 49);
	ShowCalcInfo(ln, cpu_time);
	puts("Did you get a value very close to 1.0?");
	Wait();

	puts("******************* Calculating Log(2.0) ************************");
	timer.Start();
	ln2 = Log(2.0);
	cpu_time = timer.Stop();
	ln2.Puts(5, 0, 10, 49);
	ShowCalcInfo(ln2, cpu_time);
	Wait();

	puts("****************** Calculating Exp(Log(2.0)) *******************");
	timer.Start();
	z = Exp(ln2);
	cpu_time = timer.Stop();
	z.Puts(5, 0, 10, 49);
	ShowCalcInfo(ln, cpu_time);
	puts("Did you get a value very close to 2.0?");
	Wait();

	puts("******************* Calculating 2^(1/3) *************************");
	puts("In first step : y = Log(2.0)/3.");
	y = ln2/3;
	y.Puts(5, 0, 10, 49);
	Wait();

	puts("In second step : x = Exp(y).");
	x = Exp(y);
	puts("****************** Result of x = 2^(1/3) ************************");
	x.Puts(5, 0, 10, 49);
	Wait();

	puts("******************* Calculating y = x^3 *************************");
	y = Dpow(x,3);
	puts("***************** Result of x^3 (x = 2^(1/3) ********************");
	y.Puts(5, 0, 10, 49);
	puts("Did you get a value very close to 2.0?");
	Wait();

	puts("************ Calculating 2^(1/3) by DpowD(x, p) *****************");
	timer.Start();
	z = DsDiv(SDouble(1.0), 3);		// z = 1/3 It needs a cast double to SDouble. Since version 2.20.
	y = DpowD(2.0, z);
	cpu_time = timer.Stop();
	printf("cpu time = %.3f\n", cpu_time);
	d = x - y;
	printf("Difference = ");	d.Puts(5, 0, 10, 49);
	puts("Did you get a very small value?");
	Wait();

	puts("***************** Calculating Exp(-2.5) *************************");
	y = Exp(-2.5);
	puts("****************** Result of Exp(-2.5) **************************");
	y.Puts(5, 0, 10, 49);
	Wait();

	puts("*************** Calculating Log(Exp(-2.5)) *********************");
	ln = Log(y);
	puts("**************** Result of Log(Exp(-2.5)) **********************");
	ln.Puts(5, 0, 10, 49);
	puts("Did you get a value very close to -2.5?");
	Wait();
	C.SetEffFig(0);
}

int main(){
	ErrorPuts(title);
	Exp1ByDpowD();
	logXSpeedTest();
	LogExpTest();
	Ending();
	return EXIT_SUCCESS;
}
