#include "sn.h"
ulong PrimeNumberUnder(ulong N);

double Simpson(double (*f)(double x), double x0, double x1, int num)
{
  double h;
  double sum = 0.0, x;
  int i;

  h = (x1 - x0);
  if(num) h /= num;

  x = x0;

  for(i=0; i<num; i++){
    sum += (*f)(x) + 4.*(*f)(x + h/2) + (*f)(x + h);
    x += h;
  }

  return sum*h/6;
}
double invLn(double x){
	return 1/log(x);
}
double PixApp(double x){
	double lix = Simpson(invLn,2,x,8); // bad precision for large x
	return lix*(1-1/sqrt(x));
}
double PixBySeries(long x){//Good precision for very large x only.
	/*        x          1       2!           r!
	 pi(x) = ---- {1 + ---- + -------- +...------- + ....}
			 ln x      ln x   (ln x)^2     (ln x)^r
	 ratio : q(r+1)/q(r) = (r+1)/ln x --> infinity
	 This is an asymptotic expansion which does not converge..
	*/
	double a = 1/log(x),sum = 1, p = a;
	int r;
	for(r=2;r<= x; r++){
		sum += p; p *= (a*r);
//cout << r << " sum=" << sum <<" p=" << p<<endl;
		if(a*x*p>1.0) break;
	}
	return x*a*sum;
}
long Pix(const long x) {
	long count = 0;
	int M = (x - 3)/2;	// 2M+3 <= x
	while(2*M + 3 > x) M--;
	bool* flag = new bool[M+1];	// sizeof(bool) = 1 (byte)

 	count++;
	for (int i = 0; i <= M; i++) flag[i] = true;
	for (int i = 0; i <= M; i++) {
		if (flag[i]) {
			int p = 2 * i + 3;
			for (int k = i + p; k <= M; k += p) flag[k] = false;
			count++;
		}
	}
	delete[] flag;
	return count;
}
long prime_count(long n) { // correct but very slow
	long i, j, cnt=0, sum;
	for (i = 1; i <= n; i++) {
		sum = 0;
		for (j = 1; j <= i; j++) if ( (i%j) == 0) sum += j;
		if ( sum == (i+1) ) cnt++;
	}
	return cnt;
}

int main() {
	NCBlock <ulong> table;
	long x;
/*
	for(n = 10; n< 1e9; n*=10){
		cout << n << " ";
		d = Simpson(invLn,2,n,8);
 		cout << d << endl;
	}
*/
	Timer timer;
	cout << "  x   pi(x)[exact]" << "  pi(x)-x/ln x" << "  pi(x) / (x / ln x)" << "  li x - pi(x)"<< "  x / pi(x)"<< endl;
	double xlnx, pix_xlnx, pix_div_xlnx, lix, lix_pi, x_div_pix, dx;
	double t1, t2;
	long pix;
	for(x = 10; x<1e9; x *= 10){// 1e10 Out of memory
		dx = (double)x;
		timer.Start();
		cout << "Pix(x)= " << Pix(x) << endl;
//		cout << "PixBySeries(x)= " << PixBySeries(x)<< endl;
		cout << "PrimeNumberUnder(x)=" << PrimeNumberUnder(x)<< endl;
		t1 = timer.Stop();
		timer.Start();
		pix = MakePrimeTable(table, x); // pi(x)[exact]
		t2 = timer.Stop();
cout << t1 << "  "<< t2<<endl;
		xlnx = dx*invLn(dx); // x/log(x)
		pix_xlnx = pix-xlnx; //pi(x)- x / ln x
		pix_div_xlnx = pix/xlnx;//pi(x) / (x / ln x)
		lix = Simpson(invLn,2,x,512);
		lix_pi= lix-pix;// li x - pi(x)
		x_div_pix=dx/pix;// x / pi(x)
		cout << x <<"  " << pix <<"  " << pix_xlnx << "  "<<pix_div_xlnx<<"  "<<lix_pi<<"  "<<x_div_pix<<endl;
	cout << "--------------------------" << endl;
	/*	d = NumberOfPrime(n);
		cout << "NumberOfPrime("<< n<< ")=" << d  << " diff=" << (d-c) << endl;

		double lix = Simpson(invLn,2,n,16);
		cout << "lix="<<lix<<"  lix - pi(x)=" << (lix-c) << endl;
		cout << "--------------------------------"<<endl;
	*/
	}
}
