/*  ScomplexTest3.cpp
This program compares SComplex class with C++ complex one.
It has been tested on gcc.

*****[attention]********************
To use acos(complex version)etc. compiler option
 gcc -D__NO_INLINE__ -std=c++11 ...
must be added for gcc version 4.8.1 to avoid
"error: '_hypot' was not declared in this scope"
See <math.h> included by <cmath>.

"This is a bug in MinGW, haven't found a fixed but using -D__NO_INLINE__ or editing math.h
 _hypot to hypot solved the issue, not the right fix but worked."

ref url:http://stackoverflow.com/questions/33749413/error-hypot-was-not-declared-in-this-scope
************************************
*/
#include "sn.h"

#include "FuncComp.h"
typedef complex<double> dcomplex;

static const char* const title = "A test program for SComplex class";

void ShowTitle(const char* func){
	cout << "****************** test of " << func << " ********************" << endl;
}
// error check  |er| << |cmp| ?
bool nearZero(const dcomplex &er, const dcomplex &cmp) {
	static const double d = 5.0e-13;
	double a=abs(cmp);
	if(a*er == 0.0) return true;
	double dr = abs(er.real()/a), di = abs(er.imag()/a); // relative error
	if(dr < d && di < d) return true;
	cerr << "error is large!"<< endl;
	cout << "The value = " << er << endl;
	exit(1);
}
void ZFuncCompareValues(SComplex (*pSCfunc)(const SComplex&), const string &SCfname,
 dcomplex (*pdcfunc)(const dcomplex&), const string &dcfname, const ForStm &F)
{
	string m("Comparing the values "), ast("*******");
	m =  ast + m + SCfname + "(z) and " + dcfname + "(z)" + ast;
	cout << m << endl;

	SComplex z, q;
	SDouble x, cf = Drand();
	int i, N = 10;
	dcomplex dz, dq, diff;
	cout << "  SComplex z(to double)       |  double function value     | difference between tow" << endl;
	for(i = F.init; i <= F.max; i += F.inc){
		if(i==F.skip) continue;
		x = 2*cf*i/(double)N;
		z = SComplex(x, cf*x);

		q = (*pSCfunc)(z);
		dz = complexD(z);
		dq = (*pdcfunc)(dz);
		diff = dq -complexD(q);
		nearZero(diff, dq); // error check
	// all values are putoutted in double
		cout << scientific << setprecision(5) << complexD(z) << "  "<< dq << "  " << diff << endl;
		if(i==F.max) {
			if(cf.Sign()> 0){
				cf = -cf;   i = F.init; continue;
			}else break;
		} 
	}
}
// polar(r, angle) returns r{cos(angle)+ i sin(angle)}
void PolarCompareValues(){
	cout << "Comparing the values Cpolar(r, angle) and polar(r, angle)" << endl;
	SComplex z;
	dcomplex zD, diff;
	SDouble r, angle, cf = Drand(), pi = Pi();
	double rD, angleD;
	int i, N = 10;
	z.SetFormat(z.END_CR | z.BracketFMT);

	cout << "    r      angle  z difference from double" <<endl;
	for(i=-N; i<= N;i++){
		if(!i) continue;
		r = abs(i)*cf;  rD = doubleD(r);
		angle = pi*i/(double)N; angleD = doubleD(angle);
		z = Cpolar(r, angle);
		zD = polar(rD, angleD);
		diff = complexD(z) - zD;
		nearZero(diff, zD); // error check
		cout.width(8);	cout.fill(' ');
		cout << setprecision(5) << rD << "  " << angleD << "  " << diff << endl;
	}
}
// arg(z) returns the argument of z.
void ArgCompareValues(){
	cout << "Comparing the values Arg(z) and arg(z)" << endl;
	SComplex z;
	dcomplex zD,diff;
	SDouble pi = Pi(), r, angle, x, y;
	double angleD;
	int i, N = 10;
	cout << "  z       angle    difference from double" << endl;

	for(i=-N; i<= N; i++) {
		r = i*pi/N;
		x = Drand()*Cos(r);	y = Drand()*Sin(r);
		z = SComplex(x,y);
		zD = complexD(z);
		angle = Arg(z);
		angleD = arg(zD);
		diff = doubleD(angle) - angleD;
		nearZero(diff, zD); // error check
		cout.width(8);	cout.fill(' ');
		cout << zD <<"  "<< angleD <<"  "<< diff << endl;
	}
}

int main(){
	RealSize C;
	C.SetEffFig(10);

	ErrorPuts(title);

	ForStm F(-10, 10, 1), Fn0(-10, 10, 1, 0);

	PolarCompareValues();	Wait();
	ArgCompareValues(); Wait();

// basic mathematical functions
// y = F(z) type
	ZFuncCompareValues(Cexp, "Cexp", exp, "exp", F);	Wait();
	ZFuncCompareValues(Clog, "Clog", log, "log", Fn0);	Wait();
	ZFuncCompareValues(Csin, "Csin", sin, "sin", F);	Wait();
	ZFuncCompareValues(Ccos, "Ccos", cos, "cos", F);	Wait();
	ZFuncCompareValues(Csinh, "Csinh", sinh, "sinh", F);	Wait();
	ZFuncCompareValues(Ccosh, "Ccosh", cosh, "cosh", F);	Wait();
	ZFuncCompareValues(Csqrt, "Csqrt", sqrt, "sqrt", F);	Wait();
// c++11 from here
	ZFuncCompareValues(Cacos, "Cacos", acos, "acos", F);	Wait();
	ZFuncCompareValues(Casin, "Casin", asin, "asin", F);	Wait();
	ZFuncCompareValues(Catan, "Catan", atan, "atan", F);	Wait();
	ZFuncCompareValues(Cacosh, "Cacosh", acosh, "acosh", F);	Wait();
	ZFuncCompareValues(Casinh, "Casinh", asinh, "asinh", F);	Wait();
	ZFuncCompareValues(Catanh, "Catan", atanh, "atanh", F);	Wait();
// c++11 end.
}
