/*  smpcmpx1.cpp  by K.Tsuru  */
/*  ScomplexTest1.cpp
This program compares SComplex class with C++ complex one.
It has been tested on gcc.

C++11 compiler is necessary below.
please add option parameter
 -std=c++0x
*/
#include "sn.h"
#include "wait.h"
#include "timer.h"

void ShowEffFig(){
	SDouble t;	
	cout << "Your effective figures in a decimal system = " << t.EffFig() << endl;
	Wait();
} 

typedef complex<double> dcomplex;

static const char* const title = "A test program for SComplex class";

void ShowTitle(const char* func){
	cout << "****************** test of " << func << " ********************" << endl;
}
bool isSmall(const SComplex& x) {
	static const SDouble err = 1.0e-11;
	SDouble a(Dabs(Re(x))), b(Dabs(Im(x)));
	return ( Dabs(Re(x)) < err ) && ( Dabs(Im(x)) < err ); 
}
void MultSpeedTest(){
	long c, Max = 10000001L;
	SComplex z(Sqrt(2.0),Pi()), r(z), p(z);
	Timer timer;
	double t1,t2;

	timer.Start();
	for(c= 0; c < Max; c++) r *= IU; // faster a little
	t1 = timer.Stop();
	cout << "r *= I time = " << t1 << endl << "result=" << r << endl;

	timer.Start();
	for(c= 0; c < Max; c++) p = MultI(p);
	t2 = timer.Stop();
	cout << "MultI(z) time = " << t2 << endl << "result=" << p << endl;

	r = z;
	timer.Start();
	for(c= 0; c < Max; c++) r =r * IU; // very slow
	t1 = timer.Stop();
	cout << "r = r*I time = " << t1 << endl << "result=" << r << endl;

	Wait();
}
void Cpow_nTest() {
	const char* fname = "Cpow(const SComplex& z, int n)";
	Timer timer;
	ShowTitle(fname);
	SComplex zs, rs, diff;
	dcomplex zd, rd;
	int ir, ii, N = 4;
	double a = sqrt(2.0), b = sqrt(3.0);

	cout << "  z = (x, y)     p       z^p value             relative error" << endl;
	zs.Real().SetFormat(8,8);
cout.precision(7);	
	timer.Start();

	for(ir = -N; ir <= N; ir++) {
		for(ii=-N; ii<= N; ii++) {
			if(ii==0 && ir ==0) continue;
			zd = dcomplex(a*ir, b*ii);
			zs = SComplex( zd.real(), zd.imag() );
			for(int p = -N; p <= N ; p++) {
				if(!p) continue;
				rs = Cpow(zs, p);
				rd = pow(zd, p);
				diff = rd/rs - 1.0;
				cout << zd <<"  "<< p << "  " << rs << "  "<< diff << endl;
				if(!isSmall(diff)){
					cerr << "error is large" << endl; Wait();
				}
			}
		}
	}
	double time = timer.Stop();
cout.precision(5);	
	cout << fname << " CPU time = " << time << "(sec)" << endl;
}
void Cpow_DTest() {
	const char* fname = "Cpow(const SComplex& x, const SDouble& y);";
	Timer timer;
	ShowTitle(fname);
	SComplex zs, rs, diff;
	dcomplex zd, rd;
	double pd;
	int ir, ii, N = 4;
	double a = sqrt(2.0), b = sqrt(3.0);

	cout << "  z = (x, y)               p       z^p value             relative error" << endl;
	zs.Real().SetFormat(8,8);
cout.precision(7);	
	timer.Start();

	for(ir = -N; ir <= N; ir++) {
		for(ii=-N; ii<= N; ii++) {
			if(ii==0 && ir ==0) continue;
			zd = dcomplex(a*ir, b*ii);
			zs = SComplex( zd.real(), zd.imag() );
			for(int p = -N; p <= N ; p++) {
				if(!p) continue;
				pd = p*a;
				rs = Cpow(zs, pd);
//				rs = Cpow(zs, (SDouble)pd);
				rd = pow(zd, pd);
				diff = rd/rs - 1.0;
				cout << zd <<"  "<< pd << "  " << rd << "  "<< diff << endl;
				if(!isSmall(diff)){
					cerr << "error is large" << endl; Wait();
				}
			}
		}
	}
	double time = timer.Stop();
cout.precision(5);	
	cout << fname << " CPU time = " << time << "(sec)" << endl;
}
void Cpow_DpowCTest() {
	const char* fname = "Cpow(const SDouble& a, const SComplex& p);";
	Timer timer;
	ShowTitle(fname);
	SComplex ps, rs, diff;
	dcomplex pd, rd;
	double  a;
	int ir, ii, N = 4;
	double c = sqrt(2.0), d = sqrt(3.0);

	cout << "  a              p = (x, y)            a^p value             relative error" << endl;
	ps.Real().SetFormat(8,8);
cout.precision(7);	
	timer.Start();

	int m= 0;
	for(ir = -N; ir <= N; ir++) {
		for(ii=-N; ii<= N; ii++) {
			if(ii==0 && ir ==0) continue;
			pd = dcomplex(c*ir, d*ii);
			ps = SComplex( pd.real(), pd.imag() );
			for(int ia = -N; ia <= N ; ia++) {
				if(!ia) continue;
				a = ia * d;
				rs = Cpow(a, ps);
				rd = pow(a, pd);
				diff = rd/rs - 1.0;
				cout << a <<"  "<< ps << "  " << rd << "  "<< diff << endl;
				m++;
				if(!isSmall(diff)){
					cerr << "error is large" << endl; Wait();
				}
			}
		}
	}
	double time = timer.Stop();
cout.precision(5);	
	cout << fname << " CPU time = " << time << "(sec)  " << m << "times operation"<< endl;
}
void Cpow_CpowCTest() {
	const char* fname = "Cpow(const SComplex& z, const SComplex& p);";
	Timer timer;
	ShowTitle(fname);
	double c = sqrt(2.0), d = sqrt(3.0);
	SComplex as, ps, rs, diff;
	dcomplex a, ad, pd, rd, C(c, d);
	int ir, ii, N = 4;

	cout << "  a              p = (x, y)            a^p value             relative error" << endl;
	ps.Real().SetFormat(8,8);
cout.precision(7);	
	timer.Start();

	int m= 0;
	for(ir = -N; ir <= N; ir++) {
		for(ii=-N; ii<= N; ii++) {
			if(ii==0 && ir ==0) continue;
			pd = dcomplex(c*ir, d*ii);
			ps = SComplex( pd.real(), pd.imag() );
			for(int ia = -N; ia <= N ; ia++) {
				if(!ia) continue;
				a = dcomplex(ia*c, ia*d);
				rs = Cpow(a, ps);
				rd = pow(a, pd);
				diff = rd/rs - 1.0;
				cout << a <<"  "<< ps << "  " << rd << "  "<< diff << endl;
				m++;
				if(!isSmall(diff)){
					cerr << "error is large" << endl; Wait();
				}
			}
		}
	}
	double time = timer.Stop();
cout.precision(5);	
	cout << fname << " CPU time = " << time << "(sec)  " << m << "times operation"<< endl;
}

void CXRDFuncTest(SDouble (*scfunc)(const SComplex&), double (*dfunc)(const dcomplex&), const char* fname) 
{
	Timer timer;
	ShowTitle(fname);
	SComplex zs;
	SDouble rs, diff;
	int ir,ii, N = 4, fc =0, lc =0;
	double a = sqrt(2.0), b = sqrt(3.0);
	double rd;
	dcomplex zd;
	cout << "  z = (x, y)            f(z) value             difference from double" << endl;
	zs.Real().SetFormat(8,8);
cout.precision(7);	
//	zs.SetFormat(zs.endCR, zs.BracketFMT);
	timer.Start();
	double Lt;
	for(ir = -N; ir <= N; ir++) {
		for(ii=-N; ii<= N; ii++) {
			if(ii==0 && ir ==0) continue;
			zd = dcomplex(a*ir, b*ii);
			rd = (*dfunc)(zd);
			zs = SComplex( zd.real(), zd.imag() );
			rs = (*scfunc)(zs);
			diff = rd - rs;
			cout << zd <<"  "<< rd << "  "<< diff << endl;
			lc++;
		}
		if(ir == 0) {
			Lt = timer.LapTime();
			fc = lc;	lc = 0;
			cout << " Lap time = " << Lt << "(sec) " << fc << " items" << endl;
			
		}
	}
	double time = timer.Stop();
cout.precision(5);	
	cout << fname << " CPU time = " << time << "(sec)" << endl;
	cout << "the first half = " << Lt << "(sec) " << fc << " items | "<< (Lt/fc) << "(sec/item)" << endl;
	cout << "second = " << (time-Lt) << "(sec) "  << lc << " items | "<< ((time-Lt)/lc) << "(sec/item)"  << endl;
}
void CXFuncTest(SComplex (*scfunc)(const SComplex&), dcomplex (*dfunc)(const dcomplex&), const char* fname) 
{
	Timer timer;
	ShowTitle(fname);
	SComplex zs, rs, diff;
	int ir,ii, N = 4, fc =0, lc =0;
	double a = sqrt(2.0), b = sqrt(3.0);
	dcomplex zd, rd;
	cout << "  z = (x, y)            f(z) value             difference from double" << endl;
	zs.Real().SetFormat(0,7);
	cout.precision(7);	
	
	timer.Start();
	double Lt;
	for(ir = -N; ir <= N; ir++) {
		for(ii=-N; ii<= N; ii++) {
			if(ii==0 && ir ==0) continue;
			zd = dcomplex(a*ir, b*ii);
			rd = (*dfunc)(zd);
			rs = (*scfunc)(zd);// convert "complex<double>" to "SComplex" by constructor
			diff = rd - rs;
			cout << zd <<"  "<< rd << "  "<< diff << endl;
			if(!isSmall(diff)){
				cerr << "error is large" << endl; Wait();
			}
			lc++;
		}
		if(ir == 0) {
			Lt = timer.LapTime();
			fc = lc; lc = 0;
			cout << " Lap time = " << Lt << "(sec)" << endl;
		}
	}
	double time = timer.Stop();
	cout.precision(5);	
	cout << fname << " CPU time = " << time << "(sec)" << endl;
	cout << "the first half = " << Lt << "(sec) " << fc << " items | "<< (Lt/fc) << "(sec/item)" << endl;
	cout << "second = " << (time-Lt) << "(sec) "  << lc << " items | "<< ((time-Lt)/lc) << "(sec/item)"  << endl;
}
int main(){
/* C++11 compiler is necessary below.*/
	RealSize C;
	C.SetEffFig(60);
	ErrorPuts(title);
//Cpow_DpowCTest();	Wait();
//Cpow_DTest();	Wait();
//Cpow_nTest();	Wait();
ShowEffFig();
	CXFuncTest(Ccos, cos, "cos(z)"); Wait();
	CXFuncTest(Csin, sin, "sin(z)"); Wait();
	CXFuncTest(Ctan, tan, "tan(z)"); Wait();
	CXFuncTest(Cacos, acos, "acos(z)"); Wait();
	CXFuncTest(Casin, asin, "asin(z)"); Wait();
	CXFuncTest(Catan, atan, "atan(z)"); Wait();
	CXFuncTest(Cacosh, acosh, "acosh(z)"); Wait();
	CXFuncTest(Casinh, asinh, "asinh(z)"); Wait();
	CXFuncTest(Catanh, atanh, "atanh(z)"); Wait();
	CXFuncTest(Clog, log, "log(z)"); Wait();
	CXFuncTest(Csqrt, sqrt, "sqrt(z)"); Wait();
	CXFuncTest(Cexp, exp, "exp(z)"); Wait();
	CXRDFuncTest(Arg, arg, "arg(z)"); Wait();
}
